/*
 * Created on 21 avr. 2004
 *
 * This class creates a Bonita process instance
 * thanks to the process parameters given in a XML file
 *
 */
package mc.formgenerator.bonita;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.util.Set;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import hero.interfaces.*;
import hero.util.HeroException;

/**
 * Given instance document with properties, initializes a project
 */
public class BonitaProjectExecutor {



	/**
	 * Default class constructor
	 *
	 */
	public BonitaProjectExecutor() {
	}



	/**
	 * Instanciate a process thanks to its data which are in a XML file.
	 * @param fichier XML file with the process information.
	 * @throws MalformedURLException
	 * @throws SAXException
	 * @throws IOException
	 * @throws LoginException
	 */
	public void startBonitaProject(File fichier)
		throws MalformedURLException, SAXException, IOException, Exception {


		//**********************************************************************
		//						FILE --> DOCUMENT
		//**********************************************************************

		//A parser is needed to read an XML file
		DOMParser parser = new DOMParser();

		//the parser parse the XML file 
		parser.parse(fichier.toURL().toString());

		//get a Document which represents the File. It's to normalyse the data manipulated
		Document document = parser.getDocument();



		//'parse' transforms a Document into a DataProcess
		DocumentParser parse = new DocumentParser();

		DataProject data = parse.parseProject(document);
		
		//wants the process name	
		String name = data.getProjectName();

		//wants the process properties
		HashMap table = data.getProjectProperties();

		//start the process
		instanciateProject(name, table);

	}






	/**
	 * Instanciate a process thanks to its data which are in a Document.
	 * @param document Document with the process information.
	 * @throws MalformedURLException
	 * @throws SAXException
	 * @throws IOException
	 * @throws LoginException
	 */
	public void startBonitaProject(Document document)
		throws MalformedURLException, SAXException, IOException, Exception {


		//'parse' transforms a Document into a DataProcess
		DocumentParser parse = new DocumentParser();

		DataProject data = parse.parseProject(document);
		
		//get the process name	
		String name = data.getProjectName();

		//get the process properties
		HashMap table = data.getProjectProperties();


		//start the process thanks to its name and properties
		instanciateProject(name, table);

	}





	/**
	 * Instanciate the process thanks to its name and its parameters which are in a XML file
	 * @param processName Name of the process to instanciate.
	 * @param table HashMap with 'key value' representing the process properties. 'Key' is the property name and 'Value' is its value.
	 * @throws LoginException
	 */
	private void instanciateProject(String processName, HashMap table) throws Exception {

		//The local home
		ProjectSessionHome projecth = null;

		//The bean
		ProjectSession project = null;

		try {

			//**************************************************************************
			//					INSTANCIATE THE PROCESS
			//**************************************************************************

			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();

			//Creation of the project
			project = projecth.create();

			//Project instanciation
			project.instantiateProject(processName);
			//project.initProject(processName);


			//**************************************************************************
			//					SET THE PROCESS PROPERTIES
			//**************************************************************************

			//All the keys are saved in a set
			Set collec = table.keySet();

			//Set properties if they exist
			if (collec.size() > 0) {

				//For all the keys we have to retrieve the associated value
				Iterator it = collec.iterator();

				//variable which will stock the property name
				String cle="";
				
				while (it.hasNext()) {

					//Getting the values
					cle = (String) it.next();

					//Set the current process property
					project.setProperty(cle, (String) table.get(cle));
				}
			}
			project.executeOnReadyHook();
			project.remove();

		} catch (Exception e) {
			try{
				if (project != null)
					project.remove();
			}catch(Exception re){
				System.out.println(re + " " + e.getMessage());
			}
			//System.out.println(e.getMessage());
			throw new Exception(e.getMessage());
		}
	}



}
